# ==============================================================================
# Copyright [2013] [Kevin Carter]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================


class ResultExceptions(object):
    def __init__(self, output):
        """
        Check the response codes that we get
        """
        self.output = output

    def _get_headers(self, resp):
        status_info = resp.getheaders()
        d_i = dict(status_info)
        return d_i

    def _resp_exp(self, resp, headers, authurl, jsonreq=None):
        """
        Non-20x status code responses.
        """
        if resp.status == 401:
            self.output('MESSAGE\t: Forced Re-authentication is happening.'
                        ' %s' % (resp.status))
            self.output('NOVA-API AUTH FAILURE ==> REQUEST: %s %s %s %s'
                        % (resp.status,
                           resp.reason,
                           jsonreq,
                           authurl))
            return self._get_headers(resp)
        if resp.status == 400 or resp.status == 503:
            self.output('Status = %s, It looks like Shits Broken ==> %s'
                        % (resp.status, jsonreq))
            return self._get_headers(resp)
        elif resp.status == 401:
            self.output('status = %s, You are not Authorized to perform this'
                        'action. Please check Credentials ==> %s'
                        % (resp.status, jsonreq))
            return self._get_headers(resp)
        elif resp.status == 409 or resp.status == 404:
            self.output('Status = %s Server was not found,'
                        ' likely Gone'
                        % (resp.status))
            return self._get_headers(resp)
        elif resp.status == 413:
            d_i = self._get_headers(resp)
            self.output('The System encountered an API limitation : %s'
                        % d_i)
            return d_i
        elif resp.status == 302 or resp.status == 500:
            self.output('NOVA-API REDIRECT -> REQUEST: %s %s %s %s'
                        'Make your request using a different Protocol.  IE'
                        'HTTPS or HTTP.'
                        % (resp.status,
                           resp.reason,
                           jsonreq,
                           authurl))
            return self._get_headers(resp)
        elif resp.status >= 300 <= 600:
            self.output('NOVA-API FAILURE -> REQUEST: %s %s %s %s'
                        % (resp.status,
                           resp.reason,
                           jsonreq,
                           authurl))
            return self._get_headers(resp)



# ==============================================================================
# Copyright [2013] [Kevin Carter]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
import httplib
import json

from bookofnova import statuscodes


class Connections(object):
    def __init__(self, m_args, output):
        """
        Open, Prep, or Close a connection to the OpenStack API
        """
        self.output = output
        self.m_args = m_args
        self.resp_exp = statuscodes.ResultExceptions(output)

    def _conn(self, url):
        """
        Setup a connection to the OpenStack API. This allows for both HTTP and
        HTTPS to be used. By default HTTP will be used unless YOU set the
        protocol to https, this is done with by setting the
        "m_args['use_https']" argument to "True"
        """
        if not self.m_args['use_https']:
            self.conn = httplib.HTTPConnection(url)
        else:
            self.conn = httplib.HTTPSConnection(url)

        if self.m_args['os_verbose']:
            self.conn.set_debuglevel(1)
        return self.conn

    def _conn_prep(self, path, endpoint_uri):
        """
        Setup a connection for the "path" provided to it. Using this method will
        set the default headers as well as use the "endpoint_url" method.
        """
        headers = {'X-Auth-Token': self.m_args['token'],
                   'Content-type': 'application/json'}
        endpoint = endpoint_uri.strip('http?s://')
        url_data = endpoint.split('/')
        url = url_data[0]
        c_path = '/%s%s' % ('/'.join(url_data[1:]), path)
        self.output('connecting to the API for %s' % path)
        self.conn = self._conn(url)
        if self.m_args['os_verbose']:
            self.output('\n%s %s\n' % (headers, c_path))
        return c_path, headers, url, self.conn

    def check_status(self, resp, headers, authurl, jsonreq=None):
        """
        Check the status of the response that we get from the API. Note
        that the response is given to the response exception handler if the
        response if anything over 300. This also will overwrite the
        "status['nova_reason']" key with the data provided by the response
        status interperater. Also Note that the "status['nova_reason']" key
        will be set to a Tuple containing the headers if the response status is
        above 300
        """
        # Status Data
        self.m_args['nova_status'] = resp.status
        self.m_args['nova_reason'] = resp.reason
        if resp.status >= 300:
            data = self.resp_exp._resp_exp(resp=resp,
                                           headers=headers,
                                           authurl=authurl,
                                           jsonreq=jsonreq)
            self.m_args['nova_resp'] = data

    def _delete_action(self, path, args):
        _cp = self._conn_prep(path,
                              endpoint_uri=args['nova_endpoint'])
        path = _cp[0]
        headers = _cp[1]
        url = _cp[2]
        conn = _cp[3]

        conn.request('DELETE', path, headers=headers)
        resp = conn.getresponse()

        # Status Data
        self.check_status(resp=resp,
                          headers=headers,
                          authurl=url,
                          jsonreq=None)
        if args['nova_status'] >= 300:
            conn.close()
            return args

        read_resp = resp.read()
        conn.close()
        args['nova_resp'] = read_resp
        return args

    def _get_action(self, path, args):
        _cp = self._conn_prep(path,
                              endpoint_uri=args['nova_endpoint'])
        path = _cp[0]
        headers = _cp[1]
        url = _cp[2]
        conn = _cp[3]

        conn.request('GET', path, headers=headers)
        resp = conn.getresponse()

        # Status Data
        self.check_status(resp=resp,
                          headers=headers,
                          authurl=url,
                          jsonreq=None)
        if args['nova_status'] >= 300:
            conn.close()
            return args
        else:
            read_resp = resp.read()
            conn.close()
            if read_resp:
                json_response = json.loads(read_resp)
            else:
                json_response = read_resp
            args['nova_resp'] = json_response
            return args

    def _post_action(self, path, args, body):
        _cp = self._conn_prep(path,
                              endpoint_uri=args['nova_endpoint'])
        path = _cp[0]
        headers = _cp[1]
        url = _cp[2]
        conn = _cp[3]

        conn.request('POST', path, body, headers=headers)
        resp = conn.getresponse()

        # Status Data
        self.check_status(resp=resp,
                          headers=headers,
                          authurl=url,
                          jsonreq=body)
        if args['nova_status'] >= 300:
            conn.close()
            return args

        read_resp = resp.read()
        conn.close()
        if read_resp:
            json_response = json.loads(read_resp)
        else:
            json_response = read_resp

        if args['os_verbose']:
            self.output(json.dumps(json_response, indent=2))
        args['nova_resp'] = json_response
        return args



# ==============================================================================
# Copyright [2013] [Kevin Carter]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
import json
import os
import traceback
import tempfile

# Local Imports
from bookofnova import connections, statuscodes


class MissingValues(Exception):
    pass


class NovaCommands(object):
    def __init__(self,
                 m_args,
                 output):
        """
        Arguments passed through the system are looked at in a dictionary format
        Here is an example argument set needed for operation :
        m_args = {
          "os_user": None,
          "os_apikey": None,
          "os_auth_url": None,
          "os_rax_auth": None,
          "os_verbose": None,
          "os_password": None,
          "os_tenant": None,
          "os_region": None,
          "os_version": 'v2.0'}

        This supports both Private and Public Clouds, not all functions will be
        available. Functions available are dependant on your service catalog,
        provider and version of OpenStack.

        "output" is provided so that you can also use a logging facility or
        you can set output to "print".
        """
        self.m_args = m_args
        self.output = output
        if 'os_rax_auth' in self.m_args:
            self.m_args['os_rax_auth'] = self.m_args['os_rax_auth'].upper()
        self.connection = connections.Connections(m_args=m_args,
                                                  output=output)

    def key_pair(self, key_name=None, key_path=tempfile.gettempdir()):
        """
        Build a Nova Key pair to use on boot for an instance.
        """
        self.output('Creating our Key File')
        path = '/os-keypairs'
        key_loc = '%s%s%s' % (key_path, os.sep, key_name)
        b_d = {"keypair": {"name": key_name}}
        pay_load = json.dumps(b_d)
        action = self.connection._post_action(path=path,
                                              args=self.m_args,
                                              body=pay_load)
        key_data = action['nova_resp']
        try:
            if 'keypair' in key_data:
                self.output('Key file has been Placed in "%s"' % key_loc)
                with open(key_loc, 'w+') as key_f:
                    key_f.write(key_data['keypair']['private_key'])
                    os.chmod(key_loc, 0400)
            return self.m_args
        except Exception, exp:
            self.output(exp)
            self.m_args['nova_status'] = False
            return self.m_args
        return self.m_args

    def key_pair_destroy(self, key_name, key_loc=None):
        """
        Delete a Key Pair if you had set one. If you provide "key_loc" the
        Method will attempt to delete the file as found on your system. This is
        an Optional Argument. To use this method YOU will NEED to provide the
        name of the key that you would like to Delete, as represented by
        "key_name".
        """
        self.output('Destroying our Key File')
        path = '/os-keypairs/%s' % key_name
        action = self.connection._delete_action(path=path, args=self.m_args)
        self.m_args = action
        if key_loc:
            if os.path.isfile(key_loc):
                os.remove(key_loc)
        return self.m_args

    def key_pair_list(self):
        path = '/os-keypairs'
        action = self.connection._get_action(path=path, args=self.m_args)
        self.m_args = action
        return self.m_args

    def list_quantum_networks(self):
        self.output('Checking to see if the network you specified Exists')
        path = '/os-networksv2'
        action = self.connection._get_action(path=path, args=self.m_args)
        self.m_args = action
        return self.m_args

    def builder(self, pay_load):
        """
        This method requires a "pay_load" which is a dictionary that has our
        servers build values in it. To build the instance you will NEED :

        "name", "imageRef", "flavorRef"

        However there are a lot of options that you can choose from. Here is
        an Example Payload :
        pay_load = {"name": "lynux for workgroups",
                    "imageRef": "12345678-abcd-0987-efgh-1234567890ab",
                    "flavorRef": "311",
                    "network_uuid": [uuid, uuid],
                    "rax_pub": True,
                    "rax_pri": True,
                    "inj_file": [{dst: src}, {dst: src}],
                    "key_name": "lynux",
                    "manual_disk": True,
                    "meta": [{key: value}, {key: value}]}

        Additional Options :

        "network_uuid" allows for Quantum Networking, IE RAX Public Cloud.
        You can specify the network you want to use. THe system will check
        the Netowrk for existance and then add it to the build. This is a
        standard list of UUIDs that you want to use for the networks.

        "rax_pub" and "rax_pri" are bolean values that allow for an instance to
        have public and private networking in the rackspace Public cloud. Bolean
        is a "True" or "False" Value. 

        "inj_file" allows for a file to be base64 encoded and injected on boot.
        this is a destination_on_new_instances=source_location variable.

        "key_name" allows for a SSH key to be injected on boot, This is an
        OPENSTACK only feature, IE NOT RAX Public Cloud.

        "manual_disk" is a bolean value to enable or disable the use of Managed
        disks for an Instance.

        "meta" is a used for metadata on an instance. To set metadata you will
        need to create a list of dictionaries where the "key" is the metadata
        key and the "value" is the metadata value.
        """
        if not 'name' in pay_load:
            raise MissingValues('No Name given when attempting to boot')
        elif not 'imageRef' in pay_load:
            raise MissingValues('No Image Refernce given when attempting'
                                ' to boot')
        elif not 'flavorRef' in pay_load:
            raise MissingValues('No Flavor Reference given when attempting'
                                ' to boot')
        body = {"server": {"name": pay_load['name'],
                           "imageRef": pay_load['imageRef'],
                           "flavorRef": pay_load['flavorRef']}}

        # if a Rackspace Cloud Server add the default Networks
        if self.m_args['rackspace_auth']:
            rax_pri = '11111111-1111-1111-1111-111111111111'
            prinet = {'uuid': rax_pri}
            rax_pub = '00000000-0000-0000-0000-000000000000'
            pubnet = {'uuid': rax_pub}

            networks = body['server']['networks'] = []
            # Allows the user to opt-out of a network
            if 'rax_pub' in pay_load or 'rax_pri' in pay_load:
                if pay_load['rax_pub']:
                    networks.append(pubnet)
                if pay_load['rax_pri']:
                    networks.append(prinet)
            else:
                for net in prinet, pubnet:
                    networks.append(net)

        # If quantum is used, and specified, use it.
        if 'network_uuid' in pay_load:
            if pay_load['network_uuid']:
                quantum_networks = self.list_quantum_networks()['nova_resp']
                if quantum_networks:
                    if not networks:
                        networks = body['server']['networks'] = []
                    for net in quantum_networks['networks']:
                        for net_uuid in pay_load['network_uuid']:
                            if net == net_uuid:
                                networks.append({'uuid': net})

        # Inject Files, This is generally limited to 5 injected files.
        if 'inj_file' in pay_load:
            if pay_load['inj_file']:
                personality = body['server']['personality'] = []
                for i_f in pay_load['inj_file']:
                    dst, src = i_f.split('=', 1)
                    loc_src = os.path.realpath(src)
                    if os.path.isfile(loc_src):
                        try:
                            with open(loc_src) as enc_src:
                                e_s = enc_src.read()
                                encode = e_s.encode('base64')
                                inj_construct = {'path': dst.encode('utf-8'),
                                                 'contents': encode}
                                personality.append(inj_construct)
                        except Exception, exp:
                            self.output(traceback.format_exc())
                            self.output(exp)

        # Use an SSH key on boot for an instance
        if 'key_name' in pay_load:
            if pay_load['key_name']:
                keys = self.key_pair_list()['nova_resp']
                for key in keys['keypairs']:
                    if pay_load['key_name'] == key['keypair']['name']:
                        os_key = {"key_name": pay_load['key_name']}
                        body['server'].update(os_key)

        # Use meta data if specified
        if 'meta' in pay_load:
            if pay_load['meta']:
                meta_dict = {}
                for m_f in pay_load['meta']:
                    key, value = m_f.split('=', 1)
                    meta_dict.update({key: value})
                body['server']['metadata'] = meta_dict

        # Set a Manual Disk if specified
        if 'manual_disk' in pay_load:
            if pay_load['manual_disk']:
                os_manaul_disk = {'diskConfig': 'MANUAL'}
                body['server'].update(os_manaul_disk)

        build_body = json.dumps(body)
        if self.m_args['os_verbose']:
            self.output('BUILD JSON DUMP\t:%s' % build_body)
        return build_body

    def booter(self, payload):
        """
        This method requires that you provide it a "name" for the server.
        """
        path = '/servers'
        action = self.connection._post_action(path=path,
                                              args=self.m_args,
                                              body=payload)
        self.m_args = action
        return self.m_args

    def confirm_revert_resize(self, server_id, confirm=True):
        """
        You can Confirm a resize of a server using this method. The method
        Confirms or reverts a resize action. By Default the method will confirm
        a resize for a server in a confirm Resize state.  However if you need to
        revert the resize action simply change "confirm" to "False" which will
        for the instance to be reverted.

        In order to confirm or revert a resize you will need to have the
        "server_id".
        """
        if confirm:
            payload = {"confirmResize": None}
        else:
            payload = {"revertResize": None}

        pay_load = json.dumps(payload)
        path = '/servers/%s/action' % server_id
        action = self.connection._post_action(path=path,
                                              args=self.m_args,
                                              body=pay_load)
        self.m_args = action
        return self.m_args

    def re_sizer(self, server_id, flavor):
        """
        You can resize a server using this method. The method allows for an
        instance to be resized with to any available size. Any flavor that you
        pass the instance will be looked up prior to the action being attempted.

        In order to resize a server you will need to have the "server_id" as
        well as the "flavor" size that you want to use.
        """
        flavors = self.flavor_list()
        for flv in flavors['nova_resp']['flavors']:
            if flv['id'] == flavor:
                flavor_link = flv['id']
        pay_load = json.dumps({"resize": {"flavorRef": flavor_link}})

        path = '/servers/%s/action' % server_id
        action = self.connection._post_action(path=path,
                                              args=self.m_args,
                                              body=pay_load)
        self.m_args = action
        return self.m_args

    def re_booter(self, server_id, hard_reboot=True):
        """
        You can reboot a server using this method. The method allows for an
        instance to be reboot with either a HARD or SOFT reboot. By default
        "hard_reboot" is set to "True" however if you want to perform a soft
        reboot YOU will need to set "hard_reboot" to False.

        This requires that the user, YOU, to provide a server UUID as
        "server_id".
        """
        if hard_reboot:
            payload = {"reboot": {"type": 'HARD'}}
        else:
            payload = {"reboot": {"type": 'SOFT'}}

        pay_load = json.dumps(payload)
        path = '/servers/%s/action' % server_id
        action = self.connection._post_action(path=path,
                                              args=self.m_args,
                                              body=pay_load)
        self.m_args = action
        return self.m_args

    def server_list(self):
        """
        List out all of the servers that are in the REGION you specified when
        you authenticated.
        """
        path = '/servers'
        action = self.connection._get_action(path=path, args=self.m_args)
        self.m_args = action
        return self.m_args

    def server_list_detail(self):
        """
        List out all of the servers that are in the REGION you specified when
        you authenticated. 
        """
        path = '/servers/detail'
        action = self.connection._get_action(path=path, args=self.m_args)
        self.m_args = action
        return self.m_args

    def server_info(self, server_id):
        """
        This method will allow you to see detailed information on a specified
        instance.

        This requires that the user, YOU, to provide a server UUID as
        "server_id".
        """
        path = '/servers/%s' % server_id
        action = self.connection._get_action(path=path, args=self.m_args)
        self.m_args = action
        return self.m_args

    def image_list(self):
        """
        List out all of the images that you have available to you in the
        Openstack API. Note that this only lists public images, there may be
        images that were made private which will not be shown with this command.
        """
        path = '/images'
        action = self.connection._get_action(path=path, args=self.m_args)
        self.m_args = action
        return self.m_args

    def flavor_list_detail(self):
        """
        List out all of the flavors that you have available to you in the
        Openstack API. Note that this only lists public flavors, there may be
        flavors that were made private which will not be shown with this
        command.
        """
        path = '/flavors/detail'
        action = self.connection._get_action(path=path, args=self.m_args)
        self.m_args = action
        return self.m_args

    def flavor_list(self):
        """
        List out all of the flavors that you have available to you in the
        Openstack API. Note that this only lists public flavors, there may be
        flavors that were made private which will not be shown with this
        command.
        """
        path = '/flavors'
        action = self.connection._get_action(path=path, args=self.m_args)
        self.m_args = action
        return self.m_args

    def server_nuker(self, server_id):
        """
        Delete an instance.

        This requires that the user, YOU, to provide a server UUID as
        "server_id".
        """
        self.output('Destroying Server ID "%s"' % server_id)
        path = '/servers/%s' % server_id
        action = self.connection._delete_action(path=path, args=self.m_args)
        self.m_args = action
        return self.m_args



# ==============================================================================
# Copyright [2013] [Kevin Carter]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
import httplib
import traceback
import json

# Local Import
from bookofnova import connections, statuscodes


class Authentication(object):
    def __init__(self, m_args, output):
        self.m_args = m_args
        self.output = output
        self.result_exception = statuscodes.ResultExceptions(output)

    def auth_set(self):
        """
        Set the authentication URL and STRING in JSON
        Looks for "os_arx_auth" in m_args. if RAX Auth is not set then the
        application will attempt to create an AUTH string based on the
        "os_region" and "os_auth_url"
        """
        if self.m_args['os_rax_auth'] == 'LON':
            self.m_args['os_region'] = self.m_args['os_rax_auth']
            self.m_args['os_auth_url'] = 'lon.identity.api.rackspacecloud.com'

        elif (self.m_args['os_rax_auth'] == 'DFW' or
              self.m_args['os_rax_auth'] == 'ORD'):
            self.m_args['os_region'] = self.m_args['os_rax_auth']
            self.m_args['os_auth_url'] = 'identity.api.rackspacecloud.com'

        else:
            if not self.m_args['os_region']:
                self.output('FAIL\t: You have to specify'
                                 ' a Region along with an Auth URL')
            elif not self.m_args['os_auth_url']:
                self.output('FAIL\t: You have to specify an Auth URL'
                                     ' along with the Region')

        if 'os_rax_auth' in self.m_args:
            if self.m_args['os_rax_auth']:
                self.m_args['rackspace_auth'] = True
                self.m_args['use_https'] = True
        else:
            if self.m_args['os_auth_url'].startswith('https'):
                self.m_args['use_https'] = True
            elif 'use_https' not in self.m_args:
                self.m_args['use_https'] = False
            else:
                self.m_args['use_https'] = False

        if self.m_args['os_apikey'] and self.m_args['os_rax_auth']:
            jsonreq = json.dumps({'auth':
                {'RAX-KSKEY:apiKeyCredentials':
                    {'username': self.m_args['os_user'],
                     'apiKey': self.m_args['os_apikey']
                     }}})
        else:
            jsonreq = json.dumps({'auth':
                {"tenantName": self.m_args['os_tenant'],
                 'passwordCredentials': {'username': self.m_args['os_user'],
                                         'password': self.m_args['os_password']
                                         }}})

        strip_url = self.m_args['os_auth_url'].strip('http?s://')
        self.m_args['os_auth_url'] = strip_url

        url_data = self.m_args['os_auth_url'].split('/')
        self.m_args['url'] = url_data[0]

        return self.m_args, jsonreq

    def os_auth(self):
        """
        Set a DC Endpoint and Authentication URL for the Open Stack environment
        Authentication can handle both HTTPS and HTTP Connections. 
        """
        # Set Auth Sting and URL
        data = self.auth_set()
        self.m_args = data[0]
        jsonreq = data[1]
        self.output(self.m_args)
        self.connection = connections.Connections(m_args=self.m_args,
                                                  output=self.output)
        conn = self.connection._conn(self.m_args['url'])

        if self.m_args['os_verbose']:
            self.output('%s\n' % self.m_args)
            self.output('REQUEST\t: %s\n' % jsonreq)

        headers = {'Content-Type': 'application/json'}
        tokenurl = '/%s/tokens' % self.m_args['os_version']
        conn.request('POST', tokenurl, jsonreq, headers)

        try:
            resp = conn.getresponse()
        except httplib.BadStatusLine, exp:
            self.output(exp)
            return False

        # Set the status Codes
        self.m_args['nova_status'] = resp.status
        self.m_args['nova_reason'] = resp.reason
        if resp.status >= 300:
            data = self.result_exception._resp_exp(resp=resp,
                                           headers=headers,
                                           authurl=self.m_args['os_auth_url'],
                                           jsonreq=jsonreq)
            self.m_args['nova_reason'] = data
            return self.m_args
        else:
            readresp = resp.read()
            conn.close()
            json_response = json.loads(readresp)
            self.parse_auth(json_response=json_response,
                            resp=resp)
            return self.m_args

    def parse_auth(self, json_response, resp):
        """
        Parse the arguments post authentication This will set the credentials
        including the token into the main dictionary. This loads Endpoints.
        To extend this module to set more enpoints simply follow the loop
        method. The serviceCatalog is used as a dictionary and everything in the
        service catalog is parsed as Key=Value. Notice that in my loops, I use
        ".upper()" which is not required, just allows the instances names to be
        uniform as I loop through them.
        """
        try:
            # loop through the Service Catalog
            for service in json_response['access']['serviceCatalog']:
                if service['name'].upper() == 'CLOUDSERVERSOPENSTACK':
                    for endpoint in service['endpoints']:
                        if endpoint['region'] == self.m_args['os_region']:
                            rax_cs = endpoint['publicURL']
                            self.m_args['nova_endpoint'] = rax_cs

                if service['name'].upper() == 'NOVA':
                    for endpoint in service['endpoints']:
                        if endpoint['region'] == self.m_args['os_region']:
                            os_nova = endpoint['publicURL']
                            self.m_args['nova_endpoint'] = os_nova

            # Set the token and Tenant ID
            tenant_id = json_response['access']['token']['tenant']['id']
            self.m_args['tenantid'] = tenant_id
            self.m_args['token'] = json_response['access']['token']['id']
        except (Exception, UserWarning):
            self.output('Shits Borke Son...')
            self.output(traceback.format_exc())

        # If this is a Rackspace Account set the Rackspace_auth flag
        if 'rackspace_auth' in self.m_args:
            if not self.m_args['rackspace_auth']:
                for ident in json_response['access']['user'].keys():
                    if ident.startswith('RAX-AUTH') is True:
                        self.m_args['rackspace_auth'] = True
                    else:
                        self.m_args['rackspace_auth'] = False

        if self.m_args['os_verbose']:
            self.output(json.dumps(json_response, indent=2))
            self.output(self.m_args)



#!/usr/bin/env python
# ==============================================================================
# Copyright [2013] [Kevin Carter]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
import setuptools
import sys
import os

if sys.version_info <= (2, 6, 0):
    sys.stderr.write("BookOfNova Presently requires Python"
                     " 2.6.0 or greater\n")
    sys.exit('Upgrade your version of python because it is VERY deprecated\n')

with open('README') as r_file:
    long_description = r_file.read()

setuptools.setup(
    name='bookofnova',
    version='0.003',
    author='Kevin Carter',
    author_email='kevin.carter@rackspace.com',
    description='Nova Action Library for Openstack Compute',
    long_description=long_description,
    license='Apache2.0',
    packages=['bookofnova'],
    url='https://github.com/cloudnull/bookofnova.git',
    classifiers=[
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Utilities',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ]
    )
